@@ -1,6 +1,6 @@
 /* enchant
  * Copyright (C) 2003, 2004 Dom Lachowicz
- * Copyright (C) 2017-2021 Reuben Thomas <rrt@sc3d.org>
+ * Copyright (C) 2017 Reuben Thomas <rrt@sc3d.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -17,8 +17,8 @@
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * In addition, as a special exception, the copyright holders
- * give permission to link the code of this program with
+ * In addition, as a special exception, Dom Lachowicz
+ * gives permission to link the code of this program with
  * non-LGPL Spelling Provider libraries (eg: a MSFT Office
  * spell checker backend) and distribute linked combinations including
  * the two.  You must obey the GNU Lesser General Public License in all
@@ -46,8 +46,8 @@
 
 #include "enchant.h"
 #include "enchant-provider.h"
-#include "debug.h"
 #include "pwl.h"
+#include "unused-parameter.h"
 #include "relocatable.h"
 #include "configmake.h"
 
@@ -161,13 +161,15 @@ enchant_get_conf_dirs (void)
 /* returns TRUE if tag is valid
  * for requires alphanumeric ASCII or underscore
  */
-static G_GNUC_PURE int
+static _GL_ATTRIBUTE_PURE int
 enchant_is_valid_dictionary_tag(const char * const tag)
 {
 	const char * it;
 	for (it = tag; *it; ++it)
-		if(!g_ascii_isalnum(*it) && *it != '_')
-			return 0;
+		{
+			if(!g_ascii_isalnum(*it) && *it != '_')
+				return 0;
+		}
 
 	return it != tag; /*empty tag invalid*/
 }
@@ -770,20 +772,18 @@ enchant_provider_is_valid(EnchantProvider * provider)
 {
 	if (provider == NULL)
 		g_warning ("EnchantProvider cannot be NULL\n");
-	else if (provider->dispose == NULL)
-		g_warning ("EnchantProvider's dispose method cannot be NULL\n");
-	else if (provider->request_dict == NULL)
-		g_warning ("EnchantProvider's request_dict method cannot be NULL\n");
-	else if (provider->dispose_dict == NULL)
-		g_warning ("EnchantProvider's dispose_dict method cannot be NULL\n");
 	else if (provider->identify == NULL)
 		g_warning ("EnchantProvider's identify method cannot be NULL\n");
 	else if (!g_utf8_validate((*provider->identify)(provider), -1, NULL))
-		g_warning ("EnchantProvider's identify method does not return valid UTF-8\n");
+		g_warning ("EnchantProvider's identify method does not return valid UTF-8.\n");
 	else if (provider->describe == NULL)
 		g_warning ("EnchantProvider's describe method cannot be NULL\n");
 	else if (!g_utf8_validate((*provider->describe)(provider), -1, NULL))
-		g_warning ("EnchantProvider's describe method does not return valid UTF-8\n");
+		g_warning ("EnchantProvider's describe method does not return valid UTF-8.\n");
+	else if (provider->dispose == NULL)
+		g_warning ("EnchantProvider's dispose method cannot be NULL\n");
+	else if (provider->dispose_dict == NULL)
+		g_warning ("EnchantProvider's dispose_dict method cannot be NULL\n");
 	else if (provider->list_dicts == NULL)
 		g_warning ("EnchantProvider's list_dicts method cannot be NULL\n");
 	else
@@ -808,7 +808,6 @@ enchant_load_providers_in_dir (EnchantBroker * broker, const char *dir_name)
 
 			size_t entry_len = strlen (dir_entry);
 			if ((entry_len > g_module_suffix_len) &&
-				dir_entry[0] != '.' && /* Skip hidden files */
 				!strcmp(dir_entry+(entry_len-g_module_suffix_len), G_MODULE_SUFFIX))
 				{
 #ifdef _WIN32
@@ -836,10 +835,14 @@ enchant_load_providers_in_dir (EnchantBroker * broker, const char *dir_name)
 										}
 								}
 							else
-								g_module_close (module);
+								{
+									g_module_close (module);
+								}
 						}
 					else
-						g_warning ("Error loading plugin: %s\n", g_module_error());
+						{
+							g_warning ("Error loading plugin: %s\n", g_module_error());
+						}
 
 					g_free (filename);
 #ifdef _WIN32
@@ -957,8 +960,10 @@ enchant_get_ordered_providers (EnchantBroker * broker, const char * const tag)
 
 	/* append providers not in the list, or from an unordered list */
 	for (GSList * iter = broker->provider_list; iter != NULL; iter = g_slist_next (iter))
-		if (!g_slist_find (list, iter->data))
-			list = g_slist_append (list, iter->data);
+		{
+			if (!g_slist_find (list, iter->data))
+				list = g_slist_append (list, iter->data);
+		}
 
 	return list;
 }
@@ -1078,19 +1083,25 @@ _enchant_broker_request_dict (EnchantBroker * broker, const char *const tag)
 	GSList * list = enchant_get_ordered_providers (broker, tag);
 	for (GSList *listIter = list; listIter != NULL; listIter = g_slist_next (listIter))
 		{
-			EnchantProvider * provider = (EnchantProvider *) listIter->data;
+			EnchantProvider * provider;
 
-			dict = (*provider->request_dict) (provider, tag);
+			provider = (EnchantProvider *) listIter->data;
 
-			if (dict)
+			if (provider->request_dict)
 				{
-					EnchantSession *session = enchant_session_new (provider, tag);
-					EnchantDictPrivateData *enchant_dict_private_data = g_new0 (EnchantDictPrivateData, 1);
-					enchant_dict_private_data->reference_count = 1;
-					enchant_dict_private_data->session = session;
-					dict->enchant_private_data = (void *)enchant_dict_private_data;
-					g_hash_table_insert (broker->dict_map, (gpointer)strdup (tag), dict);
-					break;
+					dict = (*provider->request_dict) (provider, tag);
+
+					if (dict)
+						{
+
+							EnchantSession *session = enchant_session_new (provider, tag);
+							EnchantDictPrivateData *enchant_dict_private_data = g_new0 (EnchantDictPrivateData, 1);
+							enchant_dict_private_data->reference_count = 1;
+							enchant_dict_private_data->session = session;
+							dict->enchant_private_data = (void *)enchant_dict_private_data;
+							g_hash_table_insert (broker->dict_map, (gpointer)strdup (tag), dict);
+							break;
+						}
 				}
 		}
 	g_slist_free (list);
@@ -1110,7 +1121,9 @@ enchant_broker_request_dict (EnchantBroker * broker, const char *const tag)
 
 	char * normalized_tag = enchant_normalize_dictionary_tag (tag);
 	if(!enchant_is_valid_dictionary_tag(normalized_tag))
-		enchant_broker_set_error (broker, "invalid tag character found");
+		{
+			enchant_broker_set_error (broker, "invalid tag character found");
+		}
 	else if ((dict = _enchant_broker_request_dict (broker, normalized_tag)) == NULL)
 		{
 			char * iso_639_only_tag = enchant_iso_639_from_tag (normalized_tag);
@@ -1143,19 +1156,13 @@ enchant_broker_describe (EnchantBroker * broker, EnchantBrokerDescribeFn fn, voi
 		}
 }
 
-static gint
-_gfunc_strcmp(gconstpointer item1, gconstpointer item2)
-{
-	return strcmp (item1, item2);
-}
-
 void
 enchant_broker_list_dicts (EnchantBroker * broker, EnchantDictDescribeFn fn, void * user_data)
 {
 	g_return_if_fail (broker);
 	g_return_if_fail (fn);
 
-	GHashTable *tag_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	GHashTable *tags = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	enchant_broker_clear_error (broker);
 
@@ -1163,50 +1170,60 @@ enchant_broker_list_dicts (EnchantBroker * broker, EnchantDictDescribeFn fn, voi
 		{
 			EnchantProvider *provider = (EnchantProvider *) list->data;
 
-			size_t n_dicts;
-			char ** dicts = (*provider->list_dicts) (provider, &n_dicts);
-
-			for (size_t i = 0; i < n_dicts; i++)
+			if (provider->list_dicts)
 				{
-					const char * tag = dicts[i];
-					if (enchant_is_valid_dictionary_tag (tag)) {
-						GSList *providers = enchant_get_ordered_providers (broker, tag);
-						gint this_priority = g_slist_index (providers, provider);
-						if (this_priority != -1) {
-							gint min_priority = this_priority + 1;
-							gpointer ptr = g_hash_table_lookup (tag_map, tag);
-							if (ptr != NULL)
-								min_priority = g_slist_index (providers, ptr);
-							if (this_priority < min_priority)
-								g_hash_table_insert (tag_map, strdup (tag), provider);
-						}
-						g_slist_free (providers);
-					}
-				}
+					size_t n_dicts;
+					char ** dicts = (*provider->list_dicts) (provider, &n_dicts);
 
-			enchant_free_string_list (dicts);
+					for (size_t i = 0; i < n_dicts; i++)
+						{
+							const char * tag;
+
+							tag = dicts[i];
+							if (enchant_is_valid_dictionary_tag (tag)) {
+								gpointer ptr;
+								GSList *providers;
+								gint this_priority;
+
+								providers = enchant_get_ordered_providers (broker, tag);
+								this_priority = g_slist_index (providers, provider);
+								if (this_priority != -1) {
+									gint min_priority;
+
+									min_priority = this_priority + 1;
+									ptr = g_hash_table_lookup (tags, tag);
+									if (ptr != NULL)
+										min_priority = g_slist_index (providers, ptr);
+									if (this_priority < min_priority)
+										g_hash_table_insert (tags, strdup (tag), provider);
+								}
+								g_slist_free (providers);
+							}
+						}
+
+					enchant_free_string_list (dicts);
+				}
 		}
 
-	GSList *tags = NULL;
 	GHashTableIter iter;
-	g_hash_table_iter_init (&iter, tag_map);
 	gpointer key, value;
+	g_hash_table_iter_init (&iter, tags);
 	while (g_hash_table_iter_next (&iter, &key, &value))
-		tags = g_slist_insert_sorted (tags, (char *) key, _gfunc_strcmp);
-
-	for (GSList *ptr = tags; ptr != NULL; ptr = g_slist_next (ptr))
 		{
-			const char *tag = (const char *) ptr->data;
-			EnchantProvider *provider = (EnchantProvider *) g_hash_table_lookup (tag_map, tag);
-			GModule *module = (GModule *) provider->enchant_private_data;
-			const char *name = (*provider->identify) (provider);
-			const char *desc = (*provider->describe) (provider);
-			const char *file = g_module_name (module);
+			const char * tag, * name, * desc, * file;
+			EnchantProvider * provider;
+			GModule *module;
+
+			tag = (const char *) key;
+			provider = (EnchantProvider *) value;
+			module = (GModule *) provider->enchant_private_data;
+			name = (*provider->identify) (provider);
+			desc = (*provider->describe) (provider);
+			file = g_module_name (module);
 			(*fn) (tag, name, desc, file, user_data);
 		}
 
-	g_slist_free (tags);
-	g_hash_table_destroy (tag_map);
+	g_hash_table_destroy (tags);
 }
 
 void
@@ -1236,16 +1253,20 @@ enchant_provider_dictionary_exists (EnchantProvider * provider, const char * con
 	int exists = 0;
 
 	if (provider->dictionary_exists)
-		exists = (*provider->dictionary_exists) (provider, tag);
-	else
+		{
+			exists = (*provider->dictionary_exists) (provider, tag);
+		}
+	else if (provider->list_dicts)
 		{
 			size_t n_dicts;
 			char ** dicts = (*provider->list_dicts) (provider, &n_dicts);
 
 			for (size_t i = 0; i < n_dicts; i++)
-				if (!strcmp(dicts[i], tag)) {
-					exists = 1;
-					break;
+				{
+					if (!strcmp(dicts[i], tag)) {
+						exists = 1;
+						break;
+					}
 				}
 
 			enchant_free_string_list (dicts);
@@ -1265,9 +1286,10 @@ _enchant_broker_dict_exists (EnchantBroker * broker, const char * const tag)
 	if (g_hash_table_lookup (broker->dict_map, (gpointer) tag) != NULL)
 		return 1;
 
-	for (GSList *list = broker->provider_list; list != NULL; list = g_slist_next (list))
+	for (GSList *list = broker->provider_list; list != NULL; list = g_slist_next (list)) {
 		if (enchant_provider_dictionary_exists ((EnchantProvider *) list->data, tag))
 			return 1;
+	}
 
 	return 0;
 }
@@ -1284,13 +1306,19 @@ enchant_broker_dict_exists (EnchantBroker * broker, const char * const tag)
 	int exists = 0;
 
 	if(!enchant_is_valid_dictionary_tag(normalized_tag))
-		enchant_broker_set_error (broker, "invalid tag character found");
+		{
+			enchant_broker_set_error (broker, "invalid tag character found");
+		}
 	else if ((exists = _enchant_broker_dict_exists (broker, normalized_tag)) == 0)
 		{
-			char * iso_639_only_tag = enchant_iso_639_from_tag (normalized_tag);
+			char * iso_639_only_tag;
+
+			iso_639_only_tag = enchant_iso_639_from_tag (normalized_tag);
 
 			if (strcmp (normalized_tag, iso_639_only_tag) != 0)
-				exists = _enchant_broker_dict_exists (broker, iso_639_only_tag);
+				{
+					exists = _enchant_broker_dict_exists (broker, iso_639_only_tag);
+				}
 
 			free (iso_639_only_tag);
 		}
@@ -1299,7 +1327,7 @@ enchant_broker_dict_exists (EnchantBroker * broker, const char * const tag)
 	return exists;
 }
 
-G_GNUC_PURE const char *
+_GL_ATTRIBUTE_PURE const char *
 enchant_dict_get_extra_word_characters (EnchantDict *dict)
 {
 	g_return_val_if_fail (dict, NULL);
@@ -1307,7 +1335,7 @@ enchant_dict_get_extra_word_characters (EnchantDict *dict)
 	return dict->get_extra_word_characters ? (*dict->get_extra_word_characters) (dict) : "";
 }
 
-G_GNUC_PURE int
+_GL_ATTRIBUTE_PURE int
 enchant_dict_is_word_character (EnchantDict * dict, uint32_t uc_in, size_t n)
 {
 	g_return_val_if_fail (n <= 2, 0);
@@ -1318,8 +1346,9 @@ enchant_dict_is_word_character (EnchantDict * dict, uint32_t uc_in, size_t n)
 	gunichar uc = (gunichar)uc_in;
 
 	/* Accept quote marks anywhere except at the end of a word */
-	if (uc == g_utf8_get_char("'") || uc == g_utf8_get_char("’"))
+	if (uc == g_utf8_get_char("'") || uc == g_utf8_get_char("’")) {
 		return n < 2;
+	}
 
 	GUnicodeType type = g_unichar_type(uc);
 
@@ -1339,8 +1368,9 @@ enchant_dict_is_word_character (EnchantDict * dict, uint32_t uc_in, size_t n)
 		return 1;     /* Enchant 1.3.0 defines word chars like this. */
 
 	case G_UNICODE_DASH_PUNCTUATION:
-		if ((n == 1) && (type == G_UNICODE_DASH_PUNCTUATION))
+		if ((n == 1) && (type == G_UNICODE_DASH_PUNCTUATION)) {
 			return 1; /* hyphens only accepted within a word. */
+		}
 		/* Fallthrough */
 
 	case G_UNICODE_CONTROL:
